@@ -100,10 +100,13 @@ typedef struct {
 	buffer *host;
 	
 	unsigned short port;
-	int usage;
 
 	time_t disable_ts;
-	
+	int is_disabled;
+	size_t balance;
+		
+	int usage; /* fair-balancing needs the no. of connections active on this host */
+	int last_used_ndx; /* round robin */
 } data_fastcgi;
 
 data_fastcgi *data_fastcgi_init(void);
