@@ -414,6 +414,20 @@ buffer_array* buffer_array_init(void) {
 	return b;
 }
 
+void buffer_array_reset(buffer_array *b) {
+	size_t i;
+	
+	if (!b) return;
+	
+	/* if they are too large, reduce them */
+	for (i = 0; i < b->used; i++) {
+		buffer_reset(b->ptr[i]);
+	}
+	
+	b->used = 0;
+}
+
+
 /**
  * free the buffer_array 
  * 
@@ -432,6 +446,7 @@ void buffer_array_free(buffer_array *b) {
 
 buffer *buffer_array_append_get_buffer(buffer_array *b) {
 	size_t i;
+	
 	if (b->size == 0) {
 		b->size = 16;
 		b->ptr = malloc(sizeof(*b->ptr) * b->size);
@@ -502,15 +517,19 @@ int buffer_is_equal(buffer *a, buffer *b) {
 	if (a->used != b->used) return 0;
 	if (a->used == 0) return 1;
 
-	for (i = a->used - 1; i < a->used && i % (sizeof(size_t)); i --) {
+	/* we are unsigned, if i < 0 it will flip to MAX_SIZE_T and will be > a->used */
+	for (i = a->used - 1; i < a->used && i % (sizeof(size_t)); i--) {
 		if (a->ptr[i] != b->ptr[i]) return 0;
 	}
 	
+	/* compare the single char itself which was kicked us out of the loop */ 
+	if (i < a->used && a->ptr[i] != b->ptr[i]) return 0;
+	
 	for (i -= (sizeof(size_t)); i < a->used; i -= (sizeof(size_t))) {
 		if (*((size_t *)(a->ptr + i)) != 
 		    *((size_t *)(b->ptr + i))) return 0;
 	}
-
+	
 	return 1;
 }
 
