@@ -75,6 +75,8 @@ static int config_insert(server *srv) {
 		{ "dir-listing.hide-dotfiles",   NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 39 */
 		{ "dir-listing.external-css",    NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },  /* 40 */
 		
+		{ "dir-listing.encoding",        NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },  /* 41 */
+		
 		{ "server.host",                 "use server.bind instead", T_CONFIG_DEPRECATED, T_CONFIG_SCOPE_UNSET },
 		{ "server.docroot",              "use server.document-root instead", T_CONFIG_DEPRECATED, T_CONFIG_SCOPE_UNSET },
 		{ "server.virtual-root",         "load mod_simple_vhost and use simple-vhost.server-root instead", T_CONFIG_DEPRECATED, T_CONFIG_SCOPE_UNSET },
@@ -91,7 +93,6 @@ static int config_insert(server *srv) {
 	/* 0 */
 	cv[0].destination = srv->srvconf.bindhost;
 	cv[1].destination = srv->srvconf.error_logfile;
-	cv[2].destination = srv->srvconf.errorfile_prefix;
 	cv[3].destination = srv->srvconf.changeroot;
 	cv[4].destination = srv->srvconf.username;
 	cv[5].destination = srv->srvconf.groupname;
@@ -106,7 +107,7 @@ static int config_insert(server *srv) {
 	cv[36].destination = &(srv->srvconf.log_request_header_on_error);
 	cv[37].destination = &(srv->srvconf.log_state_handling);
 	
-	srv->config_storage = malloc(srv->config_context->used * sizeof(specific_config *));
+	srv->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 
 	assert(srv->config_storage);
 	
@@ -125,7 +126,9 @@ static int config_insert(server *srv) {
 		s->ssl_ca_file   = buffer_init();
 		s->error_handler = buffer_init();
 		s->server_tag    = buffer_init();
+		s->errorfile_prefix = buffer_init();
 		s->dirlist_css   = buffer_init();
+		s->dirlist_encoding = buffer_init();
 		s->max_keep_alive_requests = 128;
 		s->max_keep_alive_idle = 30;
 		s->max_read_idle = 60;
@@ -140,6 +143,8 @@ static int config_insert(server *srv) {
 		s->global_bytes_per_second_cnt = 0;
 		s->global_bytes_per_second_cnt_ptr = &s->global_bytes_per_second_cnt;
 		
+		cv[2].destination = s->errorfile_prefix;
+		
 		cv[7].destination = s->server_tag;
 		cv[8].destination = &(s->use_ipv6);
 		
@@ -173,6 +178,7 @@ static int config_insert(server *srv) {
 		cv[38].destination = s->ssl_ca_file;
 		cv[39].destination = &(s->hide_dotfiles);
 		cv[40].destination = s->dirlist_css;
+		cv[41].destination = s->dirlist_encoding;
 		
 		srv->config_storage[i] = s;
 	
@@ -195,6 +201,7 @@ int config_setup_connection(server *srv, connection *con) {
 	PATCH(document_root);
 	PATCH(dir_listing);
 	PATCH(dirlist_css);
+	PATCH(dirlist_encoding);
 	PATCH(hide_dotfiles);
 	PATCH(indexfiles);
 	PATCH(max_keep_alive_requests);
@@ -203,11 +210,13 @@ int config_setup_connection(server *srv, connection *con) {
 	PATCH(max_write_idle);
 	PATCH(use_xattr);
 	PATCH(error_handler);
+	PATCH(errorfile_prefix);
 	PATCH(follow_symlink);
 	PATCH(server_tag);
 	PATCH(kbytes_per_second);
 	PATCH(global_kbytes_per_second);
 	PATCH(global_bytes_per_second_cnt);
+	
 	con->conf.global_bytes_per_second_cnt_ptr = &s->global_bytes_per_second_cnt;
 	buffer_copy_string_buffer(con->server_name, s->server_name);
 	
@@ -245,8 +254,12 @@ int config_patch_connection(server *srv, connection *con, const char *stage, siz
 				PATCH(hide_dotfiles);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("dir-listing.external-css"))) {
 				PATCH(dirlist_css);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("dir-listing.encoding"))) {
+				PATCH(dirlist_encoding);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("server.error-handler-404"))) {
 				PATCH(error_handler);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("server.errorfile-prefix"))) {
+				PATCH(errorfile_prefix);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("server.indexfiles"))) {
 				PATCH(indexfiles);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("mimetype.assign"))) {
