@@ -474,30 +474,25 @@ static handler_t cgi_connection_close(server *srv, handler_ctx *hctx) {
 
 #ifndef __WIN32
 	
-#if 0
-	log_error_write(srv, __FILE__, __LINE__, "sdd", 
-			"emergency exit: cgi", 
-			con->fd,
-			hctx->fd);
-#endif
-	
 	/* the connection to the browser went away, but we still have a connection
 	 * to the CGI script 
 	 *
 	 * close cgi-connection
 	 */
 	
-	/* close connection to the cgi-script */
-	fdevent_event_del(srv->ev, &(hctx->fde_ndx), hctx->fd);
-	fdevent_unregister(srv->ev, hctx->fd);
-	
-	if (close(hctx->fd)) {
-		log_error_write(srv, __FILE__, __LINE__, "sds", "cgi close failed ", hctx->fd, strerror(errno));
+	if (hctx->fd != -1) {
+		/* close connection to the cgi-script */
+		fdevent_event_del(srv->ev, &(hctx->fde_ndx), hctx->fd);
+		fdevent_unregister(srv->ev, hctx->fd);
+		
+		if (close(hctx->fd)) {
+			log_error_write(srv, __FILE__, __LINE__, "sds", "cgi close failed ", hctx->fd, strerror(errno));
+		}
+		
+		hctx->fd = -1;
+		hctx->fde_ndx = -1;
 	}
 	
-	hctx->fd = -1;
-	hctx->fde_ndx = -1;
-	
 	pid = hctx->pid;
 	
 	con->plugin_ctx[p->id] = NULL;
@@ -534,6 +529,13 @@ static handler_t cgi_connection_close(server *srv, handler_ctx *hctx) {
 			log_error_write(srv, __FILE__, __LINE__, "ss", "waitpid failed: ", strerror(errno));
 			return HANDLER_ERROR;
 		default:
+			/* Send an error if we haven't sent any data yet */
+			if (0 == con->file_started) {
+				connection_set_state(srv, con, CON_STATE_HANDLE_REQUEST);
+				con->http_status = 500;
+				con->mode = DIRECT;
+			}
+				
 			if (WIFEXITED(status)) {
 #if 0
 				log_error_write(srv, __FILE__, __LINE__, "sd", "(debug) cgi exited fine, pid:", pid);
@@ -588,8 +590,10 @@ static handler_t cgi_handle_fdevent(void *s, void *ctx, int revents) {
 #if 0
 			log_error_write(srv, __FILE__, __LINE__, "ddss", con->fd, hctx->fd, connection_get_state(con->state), "finished");
 #endif
+			cgi_connection_close(srv, hctx);
 			
-			break;
+			/* if we get a IN|HUP and have read everything don't exec the close twice */ 
+			return HANDLER_FINISHED;
 		case FDEVENT_HANDLED_ERROR:
 			connection_set_state(srv, con, CON_STATE_HANDLE_REQUEST);
 			con->http_status = 500;
@@ -737,6 +741,11 @@ static int cgi_create_env(server *srv, connection *con, plugin_data *p, buffer *
 		/* not needed */
 		close(to_cgi_fds[1]);
 		
+		if (srv->log_error_fd >= 0) {
+			close(STDERR_FILENO);
+			dup2(srv->log_error_fd, STDERR_FILENO);
+		}
+		
 		/* create environment */
 		env.ptr = NULL;
 		env.size = 0;
