@@ -22,6 +22,7 @@
 #include "plugin.h"
 
 #include "inet_ntop_cache.h"
+#include "crc32.h"
 
 #include <stdio.h>
 
@@ -53,24 +54,42 @@
  *            - persistent connection with upstream servers
  *            - HTTP/1.1
  */
+typedef enum {
+	PROXY_BALANCE_UNSET,
+	PROXY_BALANCE_FAIR,
+	PROXY_BALANCE_HASH,
+	PROXY_BALANCE_RR
+} proxy_balance_t;
 
 typedef struct {
 	array *extensions;
 	int debug;
+
+	proxy_balance_t balance;
 } plugin_config;
 
 typedef struct {
 	PLUGIN_DATA;
 	
 	buffer *parse_response;
+	buffer *balance_buf;
 	
 	plugin_config **config_storage;
 	
 	plugin_config conf;
 } plugin_data;
 
-typedef enum { PROXY_STATE_INIT, PROXY_STATE_CONNECT, PROXY_STATE_PREPARE_WRITE, PROXY_STATE_WRITE, PROXY_STATE_READ, PROXY_STATE_ERROR } proxy_connection_state_t;
+typedef enum { 
+	PROXY_STATE_INIT, 
+	PROXY_STATE_CONNECT, 
+	PROXY_STATE_PREPARE_WRITE, 
+	PROXY_STATE_WRITE, 
+	PROXY_STATE_READ, 
+	PROXY_STATE_ERROR 
+} proxy_connection_state_t;
+
 enum { PROXY_STDOUT, PROXY_END_REQUEST };
+
 typedef struct {
 	proxy_connection_state_t state;
 	time_t state_timestamp;
@@ -131,6 +150,7 @@ INIT_FUNC(mod_proxy_init) {
 	p = calloc(1, sizeof(*p));
 	
 	p->parse_response = buffer_init();
+	p->balance_buf = buffer_init();
 	
 	return p;
 }
@@ -142,6 +162,7 @@ FREE_FUNC(mod_proxy_free) {
 	UNUSED(srv);
 
 	buffer_free(p->parse_response);
+	buffer_free(p->balance_buf);
 	
 	if (p->config_storage) {
 		size_t i;
@@ -171,6 +192,7 @@ SETDEFAULTS_FUNC(mod_proxy_set_defaults) {
 	config_values_t cv[] = { 
 		{ "proxy.server",              NULL, T_CONFIG_LOCAL, T_CONFIG_SCOPE_CONNECTION },       /* 0 */
 		{ "proxy.debug",               NULL, T_CONFIG_SHORT, T_CONFIG_SCOPE_CONNECTION },       /* 1 */
+		{ "proxy.balance",             NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },      /* 2 */
 		{ NULL,                        NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 	};
 	
@@ -186,6 +208,9 @@ SETDEFAULTS_FUNC(mod_proxy_set_defaults) {
 		
 		cv[0].destination = s->extensions;
 		cv[1].destination = &(s->debug);
+		cv[2].destination = p->balance_buf;
+
+		buffer_reset(p->balance_buf);
 		
 		p->config_storage[i] = s;
 		ca = ((data_config *)srv->config_context->data[i])->value;
@@ -194,6 +219,20 @@ SETDEFAULTS_FUNC(mod_proxy_set_defaults) {
 			return HANDLER_ERROR;
 		}
 	
+		if (buffer_is_empty(p->balance_buf)) {
+			s->balance = PROXY_BALANCE_FAIR;
+		} else if (buffer_is_equal_string(p->balance_buf, CONST_STR_LEN("fair"))) {
+			s->balance = PROXY_BALANCE_FAIR;
+		} else if (buffer_is_equal_string(p->balance_buf, CONST_STR_LEN("round-robin"))) {
+			s->balance = PROXY_BALANCE_RR;
+		} else if (buffer_is_equal_string(p->balance_buf, CONST_STR_LEN("hash"))) {
+			s->balance = PROXY_BALANCE_HASH;
+		} else {
+			log_error_write(srv, __FILE__, __LINE__, "sb", 
+				        "proxy.balance has to be one of: fair, round-robin, hash, but not:", p->balance_buf);
+			return HANDLER_ERROR;
+		}
+
 		if (NULL != (du = array_get_element(ca, "proxy.server"))) {
 			size_t j;
 			data_array *da = (data_array *)du;
@@ -239,6 +278,7 @@ SETDEFAULTS_FUNC(mod_proxy_set_defaults) {
 					config_values_t pcv[] = { 
 						{ "host",              NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },      /* 0 */
 						{ "port",              NULL, T_CONFIG_SHORT, T_CONFIG_SCOPE_CONNECTION },       /* 1 */
+						{ "balance",              NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },   /* 2 */
 						{ NULL,                NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 					};
 					
@@ -253,6 +293,8 @@ SETDEFAULTS_FUNC(mod_proxy_set_defaults) {
 					
 					df = data_proxy_init();
 					
+					df->port = 80;
+					
 					buffer_copy_string_buffer(df->key, da_host->key);
 					
 					pcv[0].destination = df->host;
@@ -271,14 +313,6 @@ SETDEFAULTS_FUNC(mod_proxy_set_defaults) {
 								"host");
 						
 						return HANDLER_ERROR;
-					} else if (df->port == 0) {
-						log_error_write(srv, __FILE__, __LINE__, "sbbbs", 
-								"missing key (short):", 
-								da->key,
-								da_ext->key,
-								da_host->key,
-								"port");
-						return HANDLER_ERROR;
 					}
 					
 					/* if extension already exists, take it */
@@ -314,6 +348,7 @@ void proxy_connection_cleanup(server *srv, handler_ctx *hctx) {
 	
 	fdevent_event_del(srv->ev, &(hctx->fde_ndx), hctx->fd);
 	fdevent_unregister(srv->ev, hctx->fd);
+
 	if (hctx->fd != -1) {
 		close(hctx->fd);
 		srv->cur_fds--;
@@ -362,8 +397,6 @@ static int proxy_establish_connection(server *srv, handler_ctx *hctx) {
 			log_error_write(srv, __FILE__, __LINE__, "sdsd", 
 					"connect failed:", proxy_fd, strerror(errno), errno);
 			
-			proxy_connection_cleanup(srv, hctx);
-			
 			return -1;
 		}
 	}
@@ -371,6 +404,7 @@ static int proxy_establish_connection(server *srv, handler_ctx *hctx) {
 		log_error_write(srv, __FILE__, __LINE__, "sd", 
 				"connect succeeded: ", proxy_fd);
 	}
+
 	return 0;
 }
 
@@ -550,6 +584,12 @@ static int proxy_demux_response(server *srv, handler_ctx *hctx) {
 		return -1;
 	}
 
+
+	if (p->conf.debug) {
+		log_error_write(srv, __FILE__, __LINE__, "sd",
+			       "proxy - have to read:", b);
+	}
+
 	if (b > 0) {
 		if (hctx->response->used == 0) {
 			/* avoid too small buffer */
@@ -631,8 +671,9 @@ static int proxy_demux_response(server *srv, handler_ctx *hctx) {
 }
 
 
-static int proxy_write_request(server *srv, handler_ctx *hctx) {
+static handler_t proxy_write_request(server *srv, handler_ctx *hctx) {
 	data_proxy *host= hctx->host;
+	plugin_data *p    = hctx->plugin_data;
 	
 	int r;
 	
@@ -645,7 +686,7 @@ static int proxy_write_request(server *srv, handler_ctx *hctx) {
 		
 		if (-1 == (hctx->fd = socket(r, SOCK_STREAM, 0))) {
 			log_error_write(srv, __FILE__, __LINE__, "ss", "socket failed: ", strerror(errno));
-			return -1;
+			return HANDLER_ERROR;
 		}
 		hctx->fde_ndx = -1;
 		
@@ -656,9 +697,7 @@ static int proxy_write_request(server *srv, handler_ctx *hctx) {
 		if (-1 == fdevent_fcntl_set(srv->ev, hctx->fd)) {
 			log_error_write(srv, __FILE__, __LINE__, "ss", "fcntl failed: ", strerror(errno));
 			
-			proxy_connection_cleanup(srv, hctx);
-			
-			return -1;
+			return HANDLER_ERROR;
 		}
 		
 		/* fall through */
@@ -688,7 +727,10 @@ static int proxy_write_request(server *srv, handler_ctx *hctx) {
 		} else {
 			int socket_error;
 			socklen_t socket_error_len = sizeof(socket_error);
-			
+		
+			/* we don't need it anymore */	
+			fdevent_event_del(srv->ev, &(hctx->fde_ndx), hctx->fd);
+
 			/* try to finish the connect() */
 			if (0 != getsockopt(hctx->fd, SOL_SOCKET, SO_ERROR, &socket_error, &socket_error_len)) {
 				log_error_write(srv, __FILE__, __LINE__, "ss", 
@@ -703,6 +745,9 @@ static int proxy_write_request(server *srv, handler_ctx *hctx) {
 				
 				return HANDLER_ERROR;
 			}
+			if (p->conf.debug) {
+				log_error_write(srv, __FILE__, __LINE__,  "s", "proxy - connect - delayed success"); 
+			}
 		}
 		
 		proxy_set_state(srv, hctx, PROXY_STATE_PREPARE_WRITE);
@@ -713,18 +758,21 @@ static int proxy_write_request(server *srv, handler_ctx *hctx) {
 		proxy_set_state(srv, hctx, PROXY_STATE_WRITE);
 		hctx->write_offset = 0;
 		
+		fdevent_event_add(srv->ev, &(hctx->fde_ndx), hctx->fd, FDEVENT_OUT);
+
 		/* fall through */
 	case PROXY_STATE_WRITE:
 		/* continue with the code after the switch */
 		if (-1 == (r = write(hctx->fd, 
 				     hctx->write_buffer->ptr + hctx->write_offset, 
 				     hctx->write_buffer->used - hctx->write_offset))) {
-			if (errno != EAGAIN) {
+			if (errno != EAGAIN &&
+			    errno != EINTR) {
 				log_error_write(srv, __FILE__, __LINE__, "ssd", "write failed:", strerror(errno), r);
 				
-				return -1;
+				return HANDLER_ERROR;
 			} else {
-				return 0;
+				return HANDLER_WAIT_FOR_EVENT;
 			}
 		}
 		
@@ -732,18 +780,21 @@ static int proxy_write_request(server *srv, handler_ctx *hctx) {
 		
 		if (hctx->write_offset == hctx->write_buffer->used) {
 			proxy_set_state(srv, hctx, PROXY_STATE_READ);
+
+			fdevent_event_del(srv->ev, &(hctx->fde_ndx), hctx->fd);
+			fdevent_event_add(srv->ev, &(hctx->fde_ndx), hctx->fd, FDEVENT_IN);
 		}
 		
-		break;
+		return HANDLER_WAIT_FOR_EVENT;
 	case PROXY_STATE_READ:
 		/* waiting for a response */
-		break;
+		return HANDLER_WAIT_FOR_EVENT;
 	default:
 		log_error_write(srv, __FILE__, __LINE__, "s", "(debug) unknown state");
-		return -1;
+		return HANDLER_ERROR;
 	}
 	
-	return 0;
+	return HANDLER_GO_ON;
 }
 
 #define PATCH(x) \
@@ -770,6 +821,8 @@ static int mod_proxy_patch_connection(server *srv, connection *con, plugin_data
 				PATCH(extensions);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("proxy.debug"))) {
 				PATCH(debug);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("proxy.balance"))) {
+				PATCH(balance);
 			}
 		}
 	}
@@ -784,12 +837,12 @@ static int mod_proxy_setup_connection(server *srv, connection *con, plugin_data
 	
 	PATCH(extensions);
 	PATCH(debug);
+	PATCH(balance);
 	
 	return 0;
 }
 #undef PATCH
 
-
 SUBREQUEST_FUNC(mod_proxy_handle_subrequest) {
 	plugin_data *p = p_d;
 	
@@ -798,7 +851,7 @@ SUBREQUEST_FUNC(mod_proxy_handle_subrequest) {
 	size_t i;
 	
 	if (NULL == hctx) return HANDLER_GO_ON;
-	
+
 	/* select the right config */
 	mod_proxy_setup_connection(srv, con, p);
 	for (i = 0; i < srv->config_patches->used; i++) {
@@ -813,19 +866,37 @@ SUBREQUEST_FUNC(mod_proxy_handle_subrequest) {
 	if (con->mode != p->id) return HANDLER_GO_ON;
 	
 	/* ok, create the request */
-	if (-1 == proxy_write_request(srv, hctx)) {
+	switch(proxy_write_request(srv, hctx)) {
+	case HANDLER_ERROR:
 		log_error_write(srv, __FILE__, __LINE__,  "sbdd", "proxy-server disabled:", 
 				host->host,
 				host->port,
 				hctx->fd);
 		
 		/* disable this server */
-		host->usage = -1;
+		host->is_disabled = 1;
 		host->disable_ts = srv->cur_ts;
 		
+		proxy_connection_cleanup(srv, hctx);
+	
+		/* reset the enviroment and restart the sub-request */	
+		buffer_reset(con->physical.path);
 		con->mode = DIRECT;
-		con->http_status = 503;
-		return HANDLER_FINISHED;
+
+		joblist_append(srv, con);
+
+		/* mis-using HANDLER_WAIT_FOR_FD to break out of the loop 
+		 * and hope that the childs will be restarted 
+		 * 
+		 */
+
+		return HANDLER_WAIT_FOR_FD;
+	case HANDLER_WAIT_FOR_EVENT:
+		return HANDLER_WAIT_FOR_EVENT;
+	case HANDLER_WAIT_FOR_FD:
+		return HANDLER_WAIT_FOR_FD;
+	default:
+		break;
 	}
 	
 	if (con->file_started == 1) {
@@ -846,13 +917,6 @@ static handler_t proxy_connection_close(server *srv, handler_ctx *hctx) {
 	
 	if (con->mode != p->id) return HANDLER_GO_ON;
 	
-	log_error_write(srv, __FILE__, __LINE__, "ssdsd", 
-			"emergency exit: proxy:", 
-			"connection-fd:", con->fd,
-			"proxy-fd:", hctx->fd);
-	
-	
-	
 	proxy_connection_cleanup(srv, hctx);
 	
 	return HANDLER_FINISHED;
@@ -865,10 +929,15 @@ static handler_t proxy_handle_fdevent(void *s, void *ctx, int revents) {
 	connection  *con  = hctx->remote_conn;
 	plugin_data *p    = hctx->plugin_data;
 	
-	joblist_append(srv, con);
 	
 	if ((revents & FDEVENT_IN) &&
 	    hctx->state == PROXY_STATE_READ) {
+
+		if (p->conf.debug) {
+			log_error_write(srv, __FILE__, __LINE__, "sd", 
+					"proxy: fdevent-in", hctx->state);
+		}
+
 		switch (proxy_demux_response(srv, hctx)) {
 		case 0:
 			break;
@@ -876,13 +945,9 @@ static handler_t proxy_handle_fdevent(void *s, void *ctx, int revents) {
 			hctx->host->usage--;
 			
 			/* we are done */
-
-			if (chunkqueue_is_empty(con->write_queue)) {
-				connection_set_state(srv, con, CON_STATE_RESPONSE_END);
-			}
-
 			proxy_connection_cleanup(srv, hctx);
 			
+			joblist_append(srv, con);
 			return HANDLER_FINISHED;
 		case -1:
 			if (con->file_started == 0) {
@@ -895,11 +960,17 @@ static handler_t proxy_handle_fdevent(void *s, void *ctx, int revents) {
 				connection_set_state(srv, con, CON_STATE_ERROR);
 			}
 			
+			joblist_append(srv, con);
 			return HANDLER_FINISHED;
 		}
 	}
 	
 	if (revents & FDEVENT_OUT) {
+		if (p->conf.debug) {
+			log_error_write(srv, __FILE__, __LINE__, "sd", 
+					"proxy: fdevent-out", hctx->state);
+		}
+
 		if (hctx->state == PROXY_STATE_CONNECT ||
 		    hctx->state == PROXY_STATE_WRITE) {
 			/* we are allowed to send something out
@@ -909,36 +980,30 @@ static handler_t proxy_handle_fdevent(void *s, void *ctx, int revents) {
 			 */
 			return mod_proxy_handle_subrequest(srv, con, p);
 		} else {
-			log_error_write(srv, __FILE__, __LINE__, "sd", "proxy: out", hctx->state);
+			log_error_write(srv, __FILE__, __LINE__, "sd", 
+					"proxy: out", hctx->state);
 		}
 	}
 	
 	/* perhaps this issue is already handled */
 	if (revents & FDEVENT_HUP) {
-		log_error_write(srv, __FILE__, __LINE__, "sbSBSDS", 
-				"error: unexpected close of proxy connection for", 
-				con->uri.path,
-				"(no proxy process on host: ", 
-				hctx->host->host,
-				", port: ", 
-				hctx->host->port,
-				" ?)" );
-		
-#ifndef USE_LINUX_SIGIO
+		if (p->conf.debug) {
+			log_error_write(srv, __FILE__, __LINE__, "sd", 
+					"proxy: fdevent-hup", hctx->state);
+		}
+
+		con->file_finished = 1;
+
 		proxy_connection_close(srv, hctx);
-# if 0
-		log_error_write(srv, __FILE__, __LINE__, "sd", "proxy-FDEVENT_HUP", con->fd);
-# endif			
-		return HANDLER_ERROR;
-#endif
+		joblist_append(srv, con);
 	} else if (revents & FDEVENT_ERR) {
-		log_error_write(srv, __FILE__, __LINE__, "s", "proxy: err");
 		/* kill all connections to the proxy process */
 		
-		proxy_connection_close(srv, hctx);
-#if 1
 		log_error_write(srv, __FILE__, __LINE__, "s", "proxy-FDEVENT_ERR");
-#endif			
+
+		joblist_append(srv, con);
+		proxy_connection_close(srv, hctx);
+
 		return HANDLER_ERROR;
 	}
 	
@@ -948,8 +1013,8 @@ static handler_t proxy_handle_fdevent(void *s, void *ctx, int revents) {
 static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p_d) {
 	plugin_data *p = p_d;
 	size_t s_len;
-	int used = -1;
-	int ndx;
+	unsigned long last_max = ~0L;
+	int ndx = -1;
 	size_t k, i;
 	buffer *fn;
 	data_array *extension = NULL;
@@ -976,7 +1041,11 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 	
 	
 	path_info_offset = 0;
-	
+
+	if (p->conf.debug) {	
+		log_error_write(srv, __FILE__, __LINE__,  "s", "proxy - start");
+	}
+
 	/* check if extension matches */
 	for (k = 0; k < p->conf.extensions->used; k++) {
 		size_t ct_len;
@@ -1005,29 +1074,104 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 		}
 	}
 	
-	/* extension doesn't match */
 	if (k == p->conf.extensions->used) {
 		return HANDLER_GO_ON;
 	}
-	
-	/* get best server */
-	for (k = 0, ndx = -1; k < extension->value->used; k++) {
-		data_proxy *host = (data_proxy *)extension->value->data[k];
-		
-		/* enable the server again, perhaps it is back again */
-		if ((host->usage == -1) &&
-		    (srv->cur_ts - host->disable_ts > PROXY_RETRY_TIMEOUT)) {
-			host->usage = 0;
+
+	if (p->conf.debug) {	
+		log_error_write(srv, __FILE__, __LINE__,  "s", "proxy - ext found");
+	}
+
+	switch(p->conf.balance) {
+	case PROXY_BALANCE_HASH:
+		/* hash balancing */
+
+		if (p->conf.debug) {
+			log_error_write(srv, __FILE__, __LINE__,  "sd", 
+					"proxy - used hash balancing, hosts:", extension->value->used);
+		}
+
+		for (k = 0, ndx = -1, last_max = ~0L; k < extension->value->used; k++) {
+			data_proxy *host = (data_proxy *)extension->value->data[k];
+			unsigned long cur_max;
+
+			if (host->is_disabled) continue;
 			
-			log_error_write(srv, __FILE__, __LINE__,  "sbd", "proxy-server re-enabled:", 
-					host->host, host->port);
+			cur_max = generate_crc32c(CONST_BUF_LEN(con->uri.path)) +
+				generate_crc32c(CONST_BUF_LEN(host->host)) + /* we can cache this */
+				generate_crc32c(CONST_BUF_LEN(con->uri.authority));
+			
+			if (p->conf.debug) {
+				log_error_write(srv, __FILE__, __LINE__,  "sbbbd", 
+						"proxy - election:",
+						con->uri.path,
+						host->host,
+						con->uri.authority,
+						cur_max);
+			}
+
+			if ((last_max == ~0L) || /* first round */
+		   	    (cur_max > last_max)) {
+				last_max = cur_max;
+
+				ndx = k;
+			}
+		}
+
+		break;
+	case PROXY_BALANCE_FAIR:
+		/* fair balancing */
+		if (p->conf.debug) {
+			log_error_write(srv, __FILE__, __LINE__,  "s", 
+					"proxy - used fair balancing");
+		}
+
+		for (k = 0, ndx = -1, last_max = ~0L; k < extension->value->used; k++) {
+			data_proxy *host = (data_proxy *)extension->value->data[k];
+		
+			if (host->is_disabled) continue;
+
+			if (host->usage < last_max) {
+				last_max = host->usage;
+			
+				ndx = k;
+			}
+		}
+
+		break;
+	case PROXY_BALANCE_RR:
+		/* round robin */
+		if (p->conf.debug) {
+			log_error_write(srv, __FILE__, __LINE__,  "s", 
+					"proxy - used round-robin balancing");
+		}
+		for (k = 0, ndx = -1, last_max = ~0L; k < extension->value->used; k++) {
+			data_proxy *host = (data_proxy *)extension->value->data[k];
+		
+			if (host->is_disabled) continue;
+
+			/* first usable ndx */
+			if (last_max == ~0L) {
+				last_max = k;
+			}
+
+			/* get next ndx */
+			if (k > host->last_used_ndx) {
+				ndx = k;
+				host->last_used_ndx = k;
+
+				break;
+			}
 		}
 		
-		if (used == -1 || host->usage < used) {
-			used = host->usage;
-			
-			ndx = k;
+		/* didn't found a higher id, wrap to the start */
+		if (ndx != -1 && last_max != ~0L) {
+			ndx = last_max;
 		}
+
+		break;
+	default:
+		break;
 	}
 	
 	/* found a server */
@@ -1054,6 +1198,12 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 		
 		con->mode = p->id;
 		
+		if (p->conf.debug) {
+			log_error_write(srv, __FILE__, __LINE__,  "sbd", 
+					"proxy - found a host",
+					host->host, host->port);
+		}
+
 		return HANDLER_GO_ON;
 	} else {
 		/* no handler found */
@@ -1068,25 +1218,47 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 	return HANDLER_GO_ON;
 }
 
-JOBLIST_FUNC(mod_proxy_handle_joblist) {
+static handler_t mod_proxy_connection_close_callback(server *srv, connection *con, void *p_d) {
 	plugin_data *p = p_d;
-	handler_ctx *hctx = con->plugin_ctx[p->id];
 	
-	if (hctx == NULL) return HANDLER_GO_ON;
+	return proxy_connection_close(srv, con->plugin_ctx[p->id]);
+}
 
-	if (hctx->fd != -1) {
-		switch (hctx->state) {
-		case PROXY_STATE_READ:
-			fdevent_event_add(srv->ev, &(hctx->fde_ndx), hctx->fd, FDEVENT_IN);
+/**
+ *
+ * the trigger re-enables the disabled connections after the timeout is over
+ *
+ * */
+
+TRIGGER_FUNC(mod_proxy_trigger) {
+	plugin_data *p = p_d;
+
+	if (p->config_storage) {
+		size_t i, n, k;
+		for (i = 0; i < srv->config_context->used; i++) {
+			plugin_config *s = p->config_storage[i];
 			
-			break;
-		case PROXY_STATE_CONNECT:
-			fdevent_event_add(srv->ev, &(hctx->fde_ndx), hctx->fd, FDEVENT_OUT);
+			if (!s) continue; 
+
+			/* get the extensions for all configs */
 			
-			break;
-		default:
-			log_error_write(srv, __FILE__, __LINE__, "sd", "unhandled proxy.state", hctx->state);
-			break;
+			for (k = 0; k < s->extensions->used; k++) {
+				data_array *extension = (data_array *)s->extensions->data[k];
+
+				/* get all hosts */
+				for (n = 0; n < extension->value->used; n++) {
+					data_proxy *host = (data_proxy *)extension->value->data[n];
+
+					if (!host->is_disabled ||
+					    srv->cur_ts - host->disable_ts < 5) continue;
+			
+					log_error_write(srv, __FILE__, __LINE__,  "sbd", 
+							"proxy - re-enabled:",
+							host->host, host->port);
+
+					host->is_disabled = 0;
+				}
+			}
 		}
 	}
 
@@ -1094,12 +1266,6 @@ JOBLIST_FUNC(mod_proxy_handle_joblist) {
 }
 
 
-static handler_t mod_proxy_connection_close_callback(server *srv, connection *con, void *p_d) {
-	plugin_data *p = p_d;
-	
-	return proxy_connection_close(srv, con->plugin_ctx[p->id]);
-}
-
 int mod_proxy_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
 	p->name         = buffer_init_string("proxy");
@@ -1111,7 +1277,7 @@ int mod_proxy_plugin_init(plugin *p) {
 	p->handle_connection_close = mod_proxy_connection_close_callback;
 	p->handle_uri_clean        = mod_proxy_check_extension;
 	p->handle_subrequest       = mod_proxy_handle_subrequest;
-	p->handle_joblist          = mod_proxy_handle_joblist;
+	p->handle_trigger       = mod_proxy_trigger;
 	
 	p->data         = NULL;
 	
