@@ -41,10 +41,21 @@ INIT_FUNC(mod_redirect_init) {
 FREE_FUNC(mod_redirect_free) {
 	plugin_data *p = p_d;
 	
-	UNUSED(srv);
-
 	if (!p) return HANDLER_GO_ON;
-	
+
+	if (p->config_storage) {
+		size_t i;
+		for (i = 0; i < srv->config_context->used; i++) {
+			plugin_config *s = p->config_storage[i];
+			
+			pcre_keyvalue_buffer_free(s->redirect);
+			
+			free(s);
+		}
+		free(p->config_storage);
+	}
+
+
 	buffer_free(p->match_buf);
 	buffer_free(p->location);
 	
