@@ -742,7 +742,15 @@ static int http_list_directory(server *srv, connection *con, buffer *dir) {
 	if (files.used) http_dirls_sort(files.ent, files.used);
 
 	out = chunkqueue_get_append_buffer(con->write_queue);
-	BUFFER_COPY_STRING_CONST(out, "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n");
+	
+	if (buffer_is_empty(con->conf.dirlist_encoding)) {
+		BUFFER_COPY_STRING_CONST(out, "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n");
+	} else {
+		BUFFER_COPY_STRING_CONST(out, "<?xml version=\"1.0\" encoding=\"");
+		buffer_append_string_buffer(out, con->conf.dirlist_encoding);
+		BUFFER_APPEND_STRING_CONST(out, "\"?>\n");
+	}
+	
 	http_list_directory_header(out, con);
 
 	/* directories */
@@ -949,7 +957,7 @@ handler_t http_response_prepare(server *srv, connection *con) {
 		buffer_path_simplify(con->uri.path, srv->tmp_buf);
 
 		if (con->conf.log_request_handling) {
-			log_error_write(srv, __FILE__, __LINE__,  "s",  "-- sanatising URI");
+			log_error_write(srv, __FILE__, __LINE__,  "s",  "-- sanitising URI");
 			log_error_write(srv, __FILE__, __LINE__,  "sb", "URI-path     : ", con->uri.path);
 		}
 
