@@ -136,7 +136,6 @@ static server *server_init(void) {
 	CLEAN(srvconf.username);
 	CLEAN(srvconf.changeroot);
 	CLEAN(srvconf.bindhost);
-	CLEAN(srvconf.errorfile_prefix);
 	CLEAN(srvconf.license);
 	CLEAN(srvconf.event_handler);
 	CLEAN(srvconf.pid_file);
@@ -216,7 +215,6 @@ static void server_free(server *srv) {
 	CLEAN(srvconf.username);
 	CLEAN(srvconf.changeroot);
 	CLEAN(srvconf.bindhost);
-	CLEAN(srvconf.errorfile_prefix);
 	CLEAN(srvconf.license);
 	CLEAN(srvconf.event_handler);
 	CLEAN(srvconf.pid_file);
@@ -235,13 +233,18 @@ static void server_free(server *srv) {
 	if (srv->config_storage) {
 		for (i = 0; i < srv->config_context->used; i++) {
 			specific_config *s = srv->config_storage[i];
+
+			if (!s) continue;
+			
 			buffer_free(s->document_root);
 			buffer_free(s->server_name);
 			buffer_free(s->server_tag);
 			buffer_free(s->ssl_pemfile);
 			buffer_free(s->ssl_ca_file);
 			buffer_free(s->error_handler);
+			buffer_free(s->errorfile_prefix);
 			buffer_free(s->dirlist_css);
+			buffer_free(s->dirlist_encoding);
 			array_free(s->indexfiles);
 			array_free(s->mimetypes);
 			
